#! python3
# -*- encoding: utf-8 -*-

__title__           = "rtsf-web"
__short_desc__      = "only for web ui test, base on rtsf"
__version__         = "1.2.4"
__autor__           = u"罗科峰"
__author_email__    = "lkf20031988@163.com"
__license__         = 'MIT'

HOME_PAGE           = "https://github.com/RockFeng0/rtsf-web"